@@ -27,6 +27,7 @@ import os
 import shutil
 import string
 import gettext
+import shlex
 import subprocess
 import filecmp
 from mate_menu.easybuttons import *
@@ -142,6 +143,8 @@ class SuggestionButton ( Gtk.Button ):
         Align1.show()
         self.add( Align1 )
         self.show()
+        self.connect( "focus-in-event", self.onFocusIn )
+        self.connect( "focus-out-event", self.onFocusOut )
 
     def set_image(self, path):
         self.image.set_from_file(path)
@@ -152,6 +155,12 @@ class SuggestionButton ( Gtk.Button ):
     def set_icon_size (self, size):
         self.image.set_pixel_size( size )
 
+    def onFocusIn( self, widget, event ):
+        self.set_state_flags( Gtk.StateFlags.PRELIGHT, False )
+
+    def onFocusOut( self, widget, event ):
+        self.unset_state_flags( Gtk.StateFlags.PRELIGHT )
+
 class pluginclass( object ):
     TARGET_TYPE_TEXT = 80
     toButton = ( Gtk.TargetEntry.new( "text/uri-list", 0, TARGET_TYPE_TEXT ), Gtk.TargetEntry.new( "text/uri-list", 0, TARGET_TYPE_TEXT ) )
@@ -237,6 +246,11 @@ class pluginclass( object ):
             self.settings.bindGSettingsEntryToVar( "bool", "do-not-filter", self, "donotfilterapps" )
             self.settings.bindGSettingsEntryToVar( "string", "search-command", self, "searchtool" )
             self.settings.bindGSettingsEntryToVar( "int", "default-tab", self, "defaultTab" )
+            self.settings.bindGSettingsEntryToVar( "bool", "enable-ddg", self, "enableddg" )
+            self.settings.bindGSettingsEntryToVar( "bool", "enable-google", self, "enablegoogle" )
+            self.settings.bindGSettingsEntryToVar( "bool", "enable-wikipedia", self, "enablewikipedia" )
+            self.settings.bindGSettingsEntryToVar( "bool", "enable-dictionary", self, "enabledictionary" )
+            self.settings.bindGSettingsEntryToVar( "bool", "enable-computer", self, "enablecomputer" )
         except Exception, detail:
             print detail
         self.currentFavCol = 0
@@ -424,6 +438,11 @@ class pluginclass( object ):
         self.categoryhoverdelay = self.settings.get( "int", "category-hover-delay")
         self.showapplicationcomments = self.settings.get( "bool", "show-application-comments")
         self.rememberFilter = self.settings.get( "bool", "remember-filter")
+        self.enableddg = self.settings.get( "bool", "enable-ddg")
+        self.enablegoogle = self.settings.get( "bool", "enable-google")
+        self.enablewikipedia = self.settings.get( "bool", "enable-wikipedia")
+        self.enabledictionary = self.settings.get( "bool", "enable-dictionary")
+        self.enablecomputer = self.settings.get( "bool", "enable-computer")
 
         self.lastActiveTab =  self.settings.get( "int", "last-active-tab")
         self.defaultTab = self.settings.get( "int", "default-tab")
@@ -496,7 +515,11 @@ class pluginclass( object ):
         self.searchEntry.connect( "changed", self.Filter )
         self.searchEntry.connect( "activate", self.Search )
         self.showAllAppsButton.connect( "clicked", lambda widget: self.changeTab( 1 ) )
+        self.showAllAppsButton.connect( "focus-in-event", self.onFocusIn )
+        self.showAllAppsButton.connect( "focus-out-event", self.onFocusOut )
         self.showFavoritesButton.connect( "clicked", lambda widget: self.changeTab( 0 ) )
+        self.showFavoritesButton.connect( "focus-in-event", self.onFocusIn )
+        self.showFavoritesButton.connect( "focus-out-event", self.onFocusOut )
         self.buildButtonList()
 
     def blockOnPopup( self, *args ):
@@ -508,6 +531,12 @@ class pluginclass( object ):
             self.mateMenuWin.stopHiding()
         return False
 
+    def onFocusIn( self, widget, event ):
+        widget.set_state_flags( Gtk.StateFlags.PRELIGHT, False )
+
+    def onFocusOut( self, widget, event ):
+        widget.unset_state_flags( Gtk.StateFlags.PRELIGHT )
+
     def focusSearchEntry( self, clear = True ):
         # grab_focus() does select all text,
         # restoring the original selection is somehow broken, so just select the end
@@ -546,20 +575,40 @@ class pluginclass( object ):
     def add_search_suggestions(self, text):
 
         text = "<b>%s</b>" % text
+        focused = False
 
-        suggestionButton = SuggestionButton("list-add", self.iconSize, "")
-        suggestionButton.connect("clicked", self.search_google)
-        suggestionButton.set_text(_("Search Google for %s") % text)
-        suggestionButton.set_image("/usr/share/mate-menu/icons/search_engines/google.ico")
-        self.applicationsBox.add(suggestionButton)
-        self.suggestions.append(suggestionButton)
+        if self.enableddg:
+            suggestionButton = SuggestionButton("list-add", self.iconSize, "")
+            suggestionButton.connect("clicked", self.search_ddg)
+            suggestionButton.set_text(_("Search DuckDuckGo for %s") % text)
+            suggestionButton.set_image("/usr/share/mate-menu/icons/search_engines/ddg.ico")
+            self.applicationsBox.add(suggestionButton)
+            if not focused:
+                self.applicationsBox.get_children()[-1].grab_focus()
+                focused = True
+            self.suggestions.append(suggestionButton)
 
-        suggestionButton = SuggestionButton("list-add", self.iconSize, "")
-        suggestionButton.connect("clicked", self.search_wikipedia)
-        suggestionButton.set_text(_("Search Wikipedia for %s") % text)
-        suggestionButton.set_image("/usr/share/mate-menu/icons/search_engines/wikipedia.ico")
-        self.applicationsBox.add(suggestionButton)
-        self.suggestions.append(suggestionButton)
+        if self.enablegoogle:
+            suggestionButton = SuggestionButton("list-add", self.iconSize, "")
+            suggestionButton.connect("clicked", self.search_google)
+            suggestionButton.set_text(_("Search Google for %s") % text)
+            suggestionButton.set_image("/usr/share/mate-menu/icons/search_engines/google.ico")
+            self.applicationsBox.add(suggestionButton)
+            if not focused:
+                self.applicationsBox.get_children()[-1].grab_focus()
+                focused = True
+            self.suggestions.append(suggestionButton)
+
+        if self.enablewikipedia:
+            suggestionButton = SuggestionButton("list-add", self.iconSize, "")
+            suggestionButton.connect("clicked", self.search_wikipedia)
+            suggestionButton.set_text(_("Search Wikipedia for %s") % text)
+            suggestionButton.set_image("/usr/share/mate-menu/icons/search_engines/wikipedia.ico")
+            self.applicationsBox.add(suggestionButton)
+            if not focused:
+                self.applicationsBox.get_children()[-1].grab_focus()
+                focused = True
+            self.suggestions.append(suggestionButton)
 
         separator = Gtk.EventBox()
         separator.add(Gtk.Separator( orientation=Gtk.Orientation.HORIZONTAL ))
@@ -571,18 +620,26 @@ class pluginclass( object ):
         self.applicationsBox.add(separator)
         self.suggestions.append(separator)
 
-        suggestionButton = SuggestionButton("list-add", self.iconSize, "")
-        suggestionButton.connect("clicked", self.search_dictionary)
-        suggestionButton.set_text(_("Lookup %s in Dictionary") % text)
-        suggestionButton.set_image("/usr/share/mate-menu/icons/dictionary.png")
-        self.applicationsBox.add(suggestionButton)
-        self.suggestions.append(suggestionButton)
+        if self.enabledictionary:
+            suggestionButton = SuggestionButton("list-add", self.iconSize, "")
+            suggestionButton.connect("clicked", self.search_dictionary)
+            suggestionButton.set_text(_("Lookup %s in Dictionary") % text)
+            suggestionButton.set_image("/usr/share/mate-menu/icons/dictionary.png")
+            self.applicationsBox.add(suggestionButton)
+            if not focused:
+                self.applicationsBox.get_children()[-1].grab_focus()
+                focused = True
+            self.suggestions.append(suggestionButton)
 
-        suggestionButton = SuggestionButton("edit-find", self.iconSize, "")
-        suggestionButton.connect("clicked", self.Search)
-        suggestionButton.set_text(_("Search Computer for %s") % text)
-        self.applicationsBox.add(suggestionButton)
-        self.suggestions.append(suggestionButton)
+        if self.enablecomputer:
+            suggestionButton = SuggestionButton("edit-find", self.iconSize, "")
+            suggestionButton.connect("clicked", self.Search)
+            suggestionButton.set_text(_("Search Computer for %s") % text)
+            self.applicationsBox.add(suggestionButton)
+            if not focused:
+                self.applicationsBox.get_children()[-1].grab_focus()
+                focused = True
+            self.suggestions.append(suggestionButton)
 
         #self.last_separator = Gtk.EventBox()
         #self.last_separator.add(Gtk.Separator( orientation=Gtk.Orientation.HORIZONTAL ))
@@ -622,11 +679,8 @@ class pluginclass( object ):
                             i.hide()
                         else:
                             shownList.append(i)
-                            
-                            #if this is the first matching item focus it
-                            if not showns:
-                                i.grab_focus()
-
+                            # Remove application from list so that we can re-add it in order
+                            self.applicationsBox.remove(i)
                             showns = True
                 if (not showns and os.path.exists("/usr/share/mate-menu/icons/mate-logo.svg")):
                     if len(text) >= 3:
@@ -638,6 +692,16 @@ class pluginclass( object ):
                 else:
                     self.current_suggestion = None
                     self.current_results = []
+                    # Sort applications by relevance, and alphabetical within that
+                    shownList = sorted(shownList, key=lambda app: app.appName)
+                    shownList = sorted(shownList, key=lambda app: app.relevance, reverse=True)
+                    focused = False
+                    for i in shownList:
+                        self.applicationsBox.add(i)
+                        if not focused:
+                            # Grab focus of the first app shown
+                            i.grab_focus()
+                            focused = True
 
                 for i in self.categoriesBox.get_children():
                     i.set_relief( Gtk.ReliefStyle.NONE )
@@ -700,16 +764,16 @@ class pluginclass( object ):
                                  Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK)
                 #i18n
 
-                desktopMenuItem = Gtk.MenuItem(_("Add to desktop"))
-                panelMenuItem = Gtk.MenuItem(_("Add to panel"))
+                desktopMenuItem = Gtk.MenuItem(label=_("Add to desktop"))
+                panelMenuItem = Gtk.MenuItem(label=_("Add to panel"))
                 separator1 = Gtk.SeparatorMenuItem()
-                insertSpaceMenuItem = Gtk.MenuItem(_("Insert space"))
-                insertSeparatorMenuItem = Gtk.MenuItem(_("Insert separator"))
+                insertSpaceMenuItem = Gtk.MenuItem(label=_("Insert space"))
+                insertSeparatorMenuItem = Gtk.MenuItem(label=_("Insert separator"))
                 separator2 = Gtk.SeparatorMenuItem()
-                launchMenuItem = Gtk.MenuItem(_("Launch"))
-                removeFromFavMenuItem = Gtk.MenuItem(_("Remove from favorites"))
+                launchMenuItem = Gtk.MenuItem(label=_("Launch"))
+                removeFromFavMenuItem = Gtk.MenuItem(label=_("Remove from favorites"))
                 separator3 = Gtk.SeparatorMenuItem()
-                propsMenuItem = Gtk.MenuItem(_("Edit properties"))
+                propsMenuItem = Gtk.MenuItem(label=_("Edit properties"))
 
                 desktopMenuItem.connect("activate", self.add_to_desktop, widget)
                 panelMenuItem.connect("activate", self.add_to_panel, widget)
@@ -732,16 +796,20 @@ class pluginclass( object ):
 
                 mTree.show_all()
                 self.mateMenuWin.stopHiding()
-                mTree.popup(None, None, None, None, event.button, event.time)
+                mTree.attach_to_widget(widget, None)
+                if (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION) >= (3, 22):
+                    mTree.popup_at_pointer(event)
+                else:
+                    mTree.popup(None, None, None, None, event.button, event.time)
             else:
                 mTree = Gtk.Menu()
                 mTree.set_events(Gdk.EventMask.POINTER_MOTION_MASK | Gdk.EventMask.POINTER_MOTION_HINT_MASK |
                                  Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK)
 
                 #i18n
-                removeMenuItem = Gtk.MenuItem(_("Remove"))
-                insertSpaceMenuItem = Gtk.MenuItem(_("Insert space"))
-                insertSeparatorMenuItem = Gtk.MenuItem(_("Insert separator"))
+                removeMenuItem = Gtk.MenuItem(label=_("Remove"))
+                insertSpaceMenuItem = Gtk.MenuItem(label=_("Insert space"))
+                insertSeparatorMenuItem = Gtk.MenuItem(label=_("Insert separator"))
                 mTree.append(removeMenuItem)
                 mTree.append(insertSpaceMenuItem)
                 mTree.append(insertSeparatorMenuItem)
@@ -751,20 +819,24 @@ class pluginclass( object ):
                 insertSpaceMenuItem.connect( "activate", self.onFavoritesInsertSpace, widget, insertBefore )
                 insertSeparatorMenuItem.connect( "activate", self.onFavoritesInsertSeparator, widget, insertBefore )
                 self.mateMenuWin.stopHiding()
-                mTree.popup(None, None, None, None, event.button, event.time)
+                mTree.attach_to_widget(widget, None)
+                if (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION) >= (3, 22):
+                    mTree.popup_at_pointer(event)
+                else:
+                    mTree.popup(None, None, None, None, event.button, event.time)
 
     def menuPopup( self, widget, event ):
         if event.button == 3:
             mTree = Gtk.Menu()
             #i18n
-            desktopMenuItem = Gtk.MenuItem(_("Add to desktop"))
-            panelMenuItem = Gtk.MenuItem(_("Add to panel"))
+            desktopMenuItem = Gtk.MenuItem(label=_("Add to desktop"))
+            panelMenuItem = Gtk.MenuItem(label=_("Add to panel"))
             separator1 = Gtk.SeparatorMenuItem()
-            favoriteMenuItem = Gtk.CheckMenuItem(_("Show in my favorites"))
-            launchMenuItem = Gtk.MenuItem(_("Launch"))
-            deleteMenuItem = Gtk.MenuItem(_("Delete from menu"))
+            favoriteMenuItem = Gtk.CheckMenuItem(label=_("Show in my favorites"))
+            launchMenuItem = Gtk.MenuItem(label=_("Launch"))
+            deleteMenuItem = Gtk.MenuItem(label=_("Delete from menu"))
             separator2 = Gtk.SeparatorMenuItem()
-            propsMenuItem = Gtk.MenuItem(_("Edit properties"))
+            propsMenuItem = Gtk.MenuItem(label=_("Edit properties"))
 
             mTree.append(desktopMenuItem)
             mTree.append(panelMenuItem)
@@ -797,19 +869,23 @@ class pluginclass( object ):
                 favoriteMenuItem.connect( "toggled", self.onAddToFavorites, widget )
 
             self.mateMenuWin.stopHiding()
-            mTree.popup(None, None, None, None, event.button, event.time)
+            mTree.attach_to_widget(widget, None)
+            if (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION) >= (3, 22):
+                mTree.popup_at_pointer(event)
+            else:
+                mTree.popup(None, None, None, None, event.button, event.time)
 
     def searchPopup( self, widget=None, event=None ):
         menu = Gtk.Menu()
 
-        menuItem = Gtk.ImageMenuItem(_("Search Google"))
+        menuItem = Gtk.ImageMenuItem(label=_("Search Google"))
         img = Gtk.Image()
         img.set_from_file('/usr/share/mate-menu/icons/search_engines/google.ico')
         menuItem.set_image(img)
         menuItem.connect("activate", self.search_google)
         menu.append(menuItem)
 
-        menuItem = Gtk.ImageMenuItem(_("Search Wikipedia"))
+        menuItem = Gtk.ImageMenuItem(label=_("Search Wikipedia"))
         img = Gtk.Image()
         img.set_from_file('/usr/share/mate-menu/icons/search_engines/wikipedia.ico')
         menuItem.set_image(img)
@@ -819,14 +895,14 @@ class pluginclass( object ):
         menuItem = Gtk.SeparatorMenuItem()
         menu.append(menuItem)
 
-        menuItem = Gtk.ImageMenuItem(_("Lookup Dictionary"))
+        menuItem = Gtk.ImageMenuItem(label=_("Lookup Dictionary"))
         img = Gtk.Image()
         img.set_from_file('/usr/share/mate-menu/icons/dictionary.png')
         menuItem.set_image(img)
         menuItem.connect("activate", self.search_dictionary)
         menu.append(menuItem)
 
-        menuItem = Gtk.ImageMenuItem(_("Search Computer"))
+        menuItem = Gtk.ImageMenuItem(label=_("Search Computer"))
         img = Gtk.Image()
         img.set_from_icon_name("edit-find", Gtk.IconSize.INVALID)
         img.set_pixel_size( self.iconSize )
@@ -837,9 +913,12 @@ class pluginclass( object ):
         menu.show_all()
 
         self.mateMenuWin.stopHiding()
-        menu.popup(None, None, None, None, event.button, event.time)
+        menu.attach_to_widget(self.searchButton, None)
+        if (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION) >= (3, 22):
+            menu.popup_at_widget(widget, Gdk.Gravity.SOUTH_WEST, Gdk.Gravity.NORTH_WEST, event)
+        else:
+            menu.popup(None, None, None, None, event.button, event.time)
 
-        #menu.attach_to_widget(self.searchButton, None)
         #menu.reposition()
         #menu.reposition()
         #self.mateMenuWin.grab()
@@ -852,6 +931,12 @@ class pluginclass( object ):
         y = rect.y + rect.height
         return (x, y, False)
 
+    def search_ddg(self, widget):
+        text = self.searchEntry.get_text()
+        text = text.replace(" ", "+")
+        subprocess.call(['xdg-open', 'https://duckduckgo.com/?q=' + text])
+        self.mateMenuWin.hide()
+
     def search_google(self, widget):
         text = self.searchEntry.get_text()
         text = text.replace(" ", "+")
@@ -896,7 +981,7 @@ class pluginclass( object ):
 
     def delete_from_menu(self, widget, desktopEntry):
         try:
-            os.system("rm \"%s\" &" % desktopEntry.desktopFile)
+            os.remove(desktopEntry.desktopFile)
         except Exception, detail:
             print detail
 
@@ -994,10 +1079,10 @@ class pluginclass( object ):
                     app_button.execute()
                     self.mateMenuWin.hide()
                     return
-            
+
             self.mateMenuWin.hide()
             fullstring = self.searchtool.replace( "%s", text )
-            os.system(fullstring + " &")
+            subprocess.Popen(shlex.split(fullstring))
 
     def SearchWithButton( self, widget, event ):
         self.Search( widget )
@@ -1126,10 +1211,11 @@ class pluginclass( object ):
                     favButton.position = position
                     self.favorites.append( favButton )
                     self.favoritesPositionOnGrid( favButton )
-                    favButton.connect( "drag-data-received", self.onFavButtonDragReorder )
-                    favButton.drag_dest_set( Gtk.DestDefaults.MOTION | Gtk.DestDefaults.HIGHLIGHT | Gtk.DestDefaults.DROP, self.fromFav, Gdk.DragAction.COPY )
-                    favButton.connect( "drag-data-get", self.onFavButtonDragReorderGet )
-                    favButton.drag_source_set( Gdk.ModifierType.BUTTON1_MASK, self.toFav, Gdk.DragAction.COPY )
+
+                    favButton.drag_source_set (Gdk.ModifierType.BUTTON1_MASK, self.toFav, Gdk.DragAction.COPY)
+                    favButton.drag_dest_set(Gtk.DestDefaults.MOTION | Gtk.DestDefaults.HIGHLIGHT | Gtk.DestDefaults.DROP, self.toFav, Gdk.DragAction.COPY)
+                    favButton.connect("drag-data-get", self.on_drag_data_get)
+                    favButton.connect("drag-data-received", self.on_drag_data_received)
                     position += 1
 
             self.favoritesSave()
@@ -1188,10 +1274,10 @@ class pluginclass( object ):
             self.favorites.append( favButton )
             self.favoritesPositionOnGrid( favButton )
 
-            favButton.connect( "drag-data-received", self.onFavButtonDragReorder )
-            favButton.drag_dest_set( Gtk.DestDefaults.MOTION | Gtk.DestDefaults.HIGHLIGHT | Gtk.DestDefaults.DROP, self.toFav, Gdk.DragAction.COPY )
-            favButton.connect( "drag-data-get", self.onFavButtonDragReorderGet )
-            favButton.drag_source_set ( Gdk.ModifierType.BUTTON1_MASK, self.toFav, Gdk.DragAction.COPY )
+            favButton.connect("drag-data-received", self.on_drag_data_received)
+            favButton.drag_dest_set(Gtk.DestDefaults.MOTION | Gtk.DestDefaults.HIGHLIGHT | Gtk.DestDefaults.DROP, self.toFav, Gdk.DragAction.COPY)
+            favButton.connect("drag-data-get", self.on_drag_data_get)
+            favButton.drag_source_set (Gdk.ModifierType.BUTTON1_MASK, self.toFav, Gdk.DragAction.COPY)
 
             if position >= 0:
                 self.favoritesReorder( favButton.position, position )
@@ -1238,15 +1324,13 @@ class pluginclass( object ):
 
         return False
 
-    def onFavButtonDragReorderGet( self, widget, context, selection, targetType, eventTime ):
-        if targetType == self.TARGET_TYPE_FAV:
-            self.drag_origin = widget.position
-            selection.set( selection.target, 8, str(widget.position))
+    def on_drag_data_received( self, widget, context, x, y, selection, info, time ):
+        if info == self.TARGET_TYPE_FAV:
+            self.favoritesReorder( int(selection.get_data()), widget.position )
 
-    def onFavButtonDragReorder( self, widget, context, x, y, selection, targetType, time  ):
+    def on_drag_data_get( self, widget, context, selection, targetType, time ):
         if targetType == self.TARGET_TYPE_FAV:
-            #self.favoritesReorder( int(selection.data), widget.position )
-            self.favoritesReorder( self.drag_origin, widget.position )
+            selection.set(Gdk.SELECTION_CLIPBOARD, 8, str(widget.position))
 
     def on_icon_theme_changed(self, theme):
         self.menuChanged (0, 0)
